
import LineSegment;
import Rectangle;
import Tuple;
import Dimensions;
import Circle;
import Actor;
import Paddle;

import java.awt.Color;
import java.awt.Graphics;

public class Ball extends Actor
   {
   /*
   Define an actor to represent a moving ball.
   */

   //this ball looks like a circle, and it has a velocity, and color
   private Circle circle;
   private Tuple velocity;
   private Color color;

   public Ball(Theater theater, Circle circle, Tuple velocity,
      Color color)
      {
      /*
      Initialize this ball with the given values.
      */

      super(theater);

      this.circle = new Circle(circle.getCenter(), circle.getRadius());
      this.velocity = new Tuple(velocity.getX(), velocity.getY());
      this.color = color;
      }

   public void act()
      {
      /*
      Move this ball by its current velocity.
      */

      moveBy(velocity.getX(), velocity.getY());
      }

   public boolean isMobile()
      {
      /*
      Report whether this ball is mobile.
      */

      return true;
      }

   public Bounds getBounds()
      {
      /*
      Report this ball's bounds.
      */

      double radius = circle.getRadius();
      double left = circle.getXCenter() - radius;
      double top = circle.getYCenter() - radius;

      return new Bounds(new Rectangle(new Tuple(left, top),
         new Dimensions(2 * radius, 2 * radius)));
      }

   public void paint(Graphics smartTablet)
      {
      /*
      Paint this ball on the given smartTablet.
      */

      int radius = (int) circle.getRadius();
      int left = (int) circle.getXCenter() - radius;
      int top = (int) circle.getYCenter() - radius;
      int diameter = 2 * radius;

      smartTablet.setColor(color);
      smartTablet.fillOval(left, top, diameter, diameter);
      }

   private LineSegment getBallVector()
      {
      /*
      Report a vector from this ball's last to its current position.
      */

      Tuple lastCenter = new Tuple(
         circle.getXCenter() - velocity.getX(),
         circle.getYCenter() - velocity.getY());
      Tuple nextCenter = new Tuple(
         circle.getXCenter(), circle.getYCenter());

      return new LineSegment(lastCenter, nextCenter);
      }

   public void handleCollision(Polygon perimeter)
      {
      /*
      Handle collision with the boundary.
      */

      LineSegment ballVector = getBallVector();

      //find the boundary that was hit
      LineSegment hitEdge = perimeter.nearestIntersection(ballVector);

      //if no intersection found, something went wrong
      if (hitEdge == null)
         return;

      //bounce and play a sound
      bounceOff(hitEdge);
      playSound("open.au");
      }

   public void handleCollision(Actor actor)
      {
      /*
      Handle collision with another actor.
      */

      //don't do anything (pass through) any other balls
      if (actor instanceof Ball)
         return;

      LineSegment ballVector = getBallVector();

      //find the boundary that was hit
      Polygon polygon = actor.getPolygon();
      LineSegment hitEdge = polygon.nearestIntersection(ballVector);

      //if no intersection found, something went wrong
      if (hitEdge == null)
         return;

      //bounce, play a sound, and destroy any hit targets
      bounceOff(hitEdge);
      if (actor instanceof Paddle)
         playSound("fire1.au");
      else
         {
         removeActor(actor);
         playSound("fire2.au");
         }
      }

   private final void bounceOff(LineSegment barrier)
      {
      /*
      Bounce this ball off a horizontal or vertical barrier.
      */

      //setup the possible x and y corrections and this ball's bounds
      double xDelta = 0; double yDelta = 0;
      Bounds bounds = getBounds();

      if (barrier.isVertical())
         {
         double barrierX = barrier.getPoint().getX();
         if (velocity.getX() > 0) //hit a wall on the right
            xDelta = -2 * (bounds.getRight() - barrierX);
         else
            xDelta = -2 * (bounds.getLeft() - barrierX);
         velocity.setX(-velocity.getX());
         }
      else
         {
         double barrierY = barrier.getPoint().getY();
         if (velocity.getY() > 0) //hit a wall below
            yDelta = -2 * (bounds.getBottom() - barrierY);
         else
            yDelta = -2 * (bounds.getTop() - barrierY);
         velocity.setY(-velocity.getY());
         }

      moveBy(xDelta, yDelta);
      }

   private final void moveBy(double x, double y)
      {
      /*
      Alter this ball's position by the given amounts.
      */

      circle.setXCenter(circle.getXCenter() + x);
      circle.setYCenter(circle.getYCenter() + y);
      }
   }
